% script to crop out a test input image and match it against a database
% Author: Ching Lik Teo (cteo@cs.umde.edu)
% Date: 01/31/2011

clear all;
close all;

% setup paths
code_root_folder = fullfile('../match_faces');
data_root_folder = fullfile(code_root_folder, 'face_data');
gallery_root_folder = fullfile(data_root_folder, 'reference');

% add paths to workspace
addpath(genpath(code_root_folder));
addpath(genpath(data_root_folder));
addpath(genpath(gallery_root_folder));

% setup parameters
input_filename = 'test_input.jpg';
crop_polygon = [191.242275042445,533.094453876627,195.374929258631,68.1708545557442,...
                544.584210526316,66.1045274476514,546.650537634409, 531.028126768534];
std_imgsize = [150,150]; % [row(height),col(width)]
                        
%% STEP 1: Crop face image from input data
disp('crop face from test image....')
% read in test image
img_test = imread(fullfile(data_root_folder, input_filename));
%display
figure(100), imagesc(img_test); title('Original Test Image');

% convert polygon region to rectangular region
crop_region = findCropRect(crop_polygon);

% crop out the desired image region
img_cropped = imcrop(img_test, crop_region);
% display
figure(101), imagesc(img_cropped); title('Cropped Test Image');

% need to resize and convert to grayscale for correct comparison
img_cropped = im2double(rgb2gray(imresize(img_cropped, std_imgsize)));

%display
figure(102), imshow(img_cropped,[]); title('Resized Test Image');

%% STEP 2: Compare cropped face with gallery
disp('compare and find closest matching face in gallery')

% get list of files in gallery folder
galfilelist = getFileList(gallery_root_folder, 'jpg');
num_ref = size(galfilelist,2);
fprintf('Num of Gallery Images: %d\n', num_ref);

% setup vector to save matching score - use NSSD (normalized sum of squared
% distance)
score_vect = zeros(size(1,num_ref));
% loop through the gallery, resize, compare with test image and save NCC
% scores
for ii=1:num_ref
    % read in gallery image
    imgaltmp = imread(fullfile(gallery_root_folder, galfilelist{ii}));
    % resize and change to grayscale
    imgaltmp = im2double(rgb2gray(imresize(imgaltmp, std_imgsize)));
    % compute NSSD
    distTmp = sqrt(sum((img_cropped(:) - imgaltmp(:)).^2)) / sqrt(sum(imgaltmp(:).^2));
    score_vect(ii) = distTmp;
    fprintf('*');
end
fprintf('done!\n');
    
% plot out NSSD scores
figure(103), plot(score_vect, 'r*-'); title('NSSD scores'), xlabel('Gallery Image ID'), ylabel('NSSD scores');
grid on;    

% get the lowest score and dislay images side by side
[val,min_id] = min(score_vect);
match_img = imread(fullfile(gallery_root_folder, galfilelist{min_id}));
match_img = im2double(rgb2gray(imresize(match_img, std_imgsize)));
concatImg = [img_cropped,match_img];
figure(104), imshow(concatImg,[]), title(sprintf('Matching Images with score: %.2f', val)); xlabel('<Test>           <Gallery>');

% write out the concatenated image to file
disp('saving results')
save_str = fullfile(data_root_folder, 'res_out.jpg');
imwrite(concatImg, save_str, 'jpg');
